package external

import (
	"bytes"
	"crypto/tls"
	"encoding/base64"
	"encoding/json"
	"fmt"
	"institutionMS/layers/entity"
	"net/http"
)

// AuthAPI is a struct that defines authentication data for external API
type AuthAPI struct {
	Domain string
	MgmtToken string
	Connection string
	ClientId string
	Secret string
}

// createPayload is a struct that defines the body for signup
type createPayload struct {
	Connection string `json:"connection"`
	Email string `json:"email"`
	Password string `json:"password"`
}

// readPayload is a struct that defines the body for login
type readPayload struct {
	Username string `json:"username"`
	Password string `json:"password"`
	Connection string `json:"connection"`
	ClientId string `json:"client_id"`
	ClientSecret string `json:"client_secret"`
	GrantType string `json:"grant_type"`
}

// updatePasswordPayload is a struct that defines the body for password reset
type updatePasswordPayload struct {
	ClientId string `json:"client_id"`
	Email string `json:"email"`
	Connection string `json:"connection"`
}

const (
	authToken         = "Bearer %s"
	passwordGrantType = "password"
)

// sendRequest is a generic function that sends request to external auth API using TLS
func (api AuthAPI) sendRequest(headers map[string]string, body interface{}, url string, method string) (res *http.Response, err error) {
	c := &http.Client{
		Transport: &http.Transport{
			TLSClientConfig: &tls.Config{InsecureSkipVerify: true},
		},
	}
	u := fmt.Sprintf(url, api.Domain)
	b, err := json.Marshal(body)
	if err != nil {
		return &http.Response{}, err
	}
	req, err := http.NewRequest(method, u, bytes.NewBuffer(b))
	if err != nil {
		return &http.Response{}, err
	}
	for k, v := range headers {
		req.Header.Set(k, v)
	}
	res, err = c.Do(req)
	return
}

// Create issues Institution's signup at external auth API
func(api AuthAPI) Create(auth entity.Auth) (res *http.Response, err error) {
	body := createPayload{
		Connection: api.Connection,
		Email:      auth.Email,
		Password:   auth.Password,
	}
	headers := map[string]string {
		"Content-Type": "application/json",
		"Authorization": fmt.Sprintf(authToken, api.MgmtToken),
	}
	res, err = api.sendRequest(headers, body, "https://%s/api/v2/users", http.MethodPost)
	return
}

// Read issues Institution's login at external auth API
func(api AuthAPI) Read(auth entity.Auth) (res *http.Response, err error) {
	secret, err := base64.URLEncoding.DecodeString(api.Secret)
	if err != nil {
		return
	}
	body := readPayload{
		Username:     auth.Email,
		Password:     auth.Password,
		Connection:   api.Connection,
		ClientId:     api.ClientId,
		ClientSecret: string(secret),
		GrantType:    passwordGrantType,
	}
	headers := map[string]string {
		"Content-Type": "application/json",
	}
	res, err = api.sendRequest(headers, body, "https://%s/oauth/token", http.MethodPost)
	return
}

// ReadByEmail issues search by Institution's email at external auth API
func(api AuthAPI) ReadByEmail(email string) (res *http.Response, err error) {
	getByEmailUrl := "https://%s/api/v2/users-by-email"
	emailQueryString := fmt.Sprintf("?email=%s", email)
	getByEmailUrl += emailQueryString
	headers := map[string]string {
		"Content-Type": "application/json",
		"Authorization": fmt.Sprintf(authToken, api.MgmtToken),
	}
	res, err = api.sendRequest(headers, nil, getByEmailUrl, http.MethodGet)
	return
}

// Create issues Institution's password reset at external auth API
func(api AuthAPI) UpdatePassword(email string) (res *http.Response, err error) {
	body := updatePasswordPayload{
		ClientId:   api.ClientId,
		Email:      email,
		Connection: api.Connection,
	}
	headers := map[string]string {
		"Content-Type": "application/json",
	}
	res, err = api.sendRequest(headers, body, "https://%s/dbconnections/change_password", http.MethodPost)
	return
}

// Create issues Institution's account deletion at external auth API
func(api AuthAPI) Delete(id string) (res *http.Response, err error) {
	headers := map[string]string {
		"Content-Type": "application/json",
		"Authorization": fmt.Sprintf(authToken, api.MgmtToken),
	}
	res, err = api.sendRequest(headers, nil, "https://%s/api/v2/users/"+id, http.MethodDelete)
	return
}
