package infrastructure

import (
	"context"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"go.mongodb.org/mongo-driver/mongo"
	"go.mongodb.org/mongo-driver/mongo/options"
	"institutionMS/layers/entity"
	"log"
	"time"
)

// MongoDB is a struct that defines a Mongo DB connection at infrastructure layer
type MongoDB struct {
	Username   string
	Password   string
	Host       string
	Port       string
	client     *mongo.Client
	ctx        context.Context
	collection *mongo.Collection
}

// Create saves a Institution at MongoDB and returns its given id
func (mongodb MongoDB) Create(institution entity.Institution) (id string, err error) {
	res, err := mongodb.collection.InsertOne(mongodb.ctx, institution)
	if err == nil {
		if res != nil {
			objId, _ := res.InsertedID.(primitive.ObjectID)
			id = objId.Hex()
		}
	}
	return
}

// Read loads a Institution from MongoDB through email
func (mongodb MongoDB) Read(email string) (institution *entity.Institution, err error) {
	err = mongodb.collection.FindOne(
		mongodb.ctx,
		bson.D{{"email", email}},
	).Decode(&institution)
	if err != nil {
		if err == mongo.ErrNoDocuments {
			err = nil
		}
	}
	return
}

// Update a Institution from MongoDB through id
func (mongodb MongoDB) Update(objId primitive.ObjectID, institution *entity.Institution) (updated *entity.Institution, err error) {
	opts := options.FindOneAndUpdate().SetReturnDocument(options.After)

	filter := bson.D{{"_id", objId}}
	update := bson.D{{"$set",
		bson.D{
			{"isValid", institution.IsValid},
			{"name", institution.Name},
			{"email", institution.Email},
			{"description", institution.Description},
			{"location", institution.Location},
		},
	}}
	err = mongodb.collection.FindOneAndUpdate(mongodb.ctx, filter, update, opts).Decode(&updated)

	if err != nil {
		if err == mongo.ErrNoDocuments {
			err = nil
		}
	}
	return
}

// Delete a Institution from MongoDB through id
func (mongodb MongoDB) Delete(objId primitive.ObjectID) (deleted *entity.Institution, err error) {
	err = mongodb.collection.FindOneAndDelete(
		mongodb.ctx,
		bson.M{"_id": objId},
		).Decode(&deleted)
	if err != nil {
		if err == mongo.ErrNoDocuments {
			err = nil
		}
	}
	return
}

// setUri sets the connection string for MongoDB connection
func (mongodb *MongoDB) setUri() (uri string) {
	uri = "mongodb://" + mongodb.Host + ":" + mongodb.Port
	return
}

// setCredentials sets credentials for MongoDB connection
func (mongodb *MongoDB) setCredentials() (credentials options.Credential) {
	credentials = options.Credential{
		Username: mongodb.Username,
		Password: mongodb.Password,
	}
	return
}

// Connect creates a connection to MongoDB
func (mongodb *MongoDB) Connect() {
	var err error

	mongodb.ctx, _ = context.WithTimeout(context.Background(), 60*time.Second)
	opts := options.Client().ApplyURI(mongodb.setUri()).SetAuth(mongodb.setCredentials())
	mongodb.client, err = mongo.Connect(mongodb.ctx, opts)

	if err != nil {
		log.Fatal(error.Error(err))
	}
}

// Collection sets a collection at a database for MongoDB connection
func (mongodb *MongoDB) Collection(database string, collection string) {
	mongodb.collection = mongodb.client.Database(database).Collection(collection)
}

// Disconnect closes MongoDB client using timeout context
func (mongodb MongoDB) Disconnect() {
	err := mongodb.client.Disconnect(mongodb.ctx)

	if err != nil {
		log.Fatal(error.Error(err))
	}
}