package service

import (
	"encoding/json"
	"fmt"
	"github.com/gin-gonic/gin"
	"institutionMS/external"
	"institutionMS/layers/entity"
	"io/ioutil"
	"net/http"
)

// AuthService is a struct that defines authentication at service layer
type AuthService struct {
	Api external.AuthAPI
}

// readSingleResponse reads a single JSON object as a map
func (service AuthService) readSingleResponse(res *http.Response) (data map[string]interface{}, err error) {
	defer res.Body.Close()
	bytes, err := ioutil.ReadAll(res.Body)
	if err == nil {
		err = json.Unmarshal(bytes, &data)
	}
	return
}

// readMultipleResponse reads an array of JSON objects as a map
func (service AuthService) readMultipleResponse(res *http.Response) (data []map[string]interface{}, err error) {
	defer res.Body.Close()
	bytes, err := ioutil.ReadAll(res.Body)
	if err == nil {
		err = json.Unmarshal(bytes, &data)
	}
	return
}

// Create defines logic for Volunteer's signup
func (service AuthService) Create(ctx *gin.Context) {
	auth := new(entity.Auth)
	err := ctx.Bind(auth)
	if err != nil {
		ctx.JSON(http.StatusBadRequest, gin.H{"error": error.Error(err)})
	} else {
		res, err := service.Api.Create(*auth)
		if err != nil {
			ctx.JSON(http.StatusInternalServerError, gin.H{"error": error.Error(err)})
		} else {
			data, err := service.readSingleResponse(res)
			if err != nil {
				ctx.JSON(http.StatusInternalServerError, gin.H{"error": error.Error(err)})
			} else {
				if res.StatusCode != http.StatusCreated {
					ctx.JSON(res.StatusCode, gin.H{"error": fmt.Sprintf("%s", data)})
				} else {
					ctx.JSON(res.StatusCode, gin.H{"email": auth.Email})
				}
			}
		}
	}
}

// Read defines logic for Volunteer's login
func (service AuthService) Read(ctx *gin.Context) {
	auth := new(entity.Auth)
	err := ctx.Bind(auth)
	if err != nil {
		ctx.JSON(http.StatusBadRequest, gin.H{"error": error.Error(err)})
	} else {
		res, err := service.Api.Read(*auth)
		if err != nil {
			ctx.JSON(http.StatusInternalServerError, gin.H{"error": error.Error(err)})
		} else {
			data, err := service.readSingleResponse(res)
			if err != nil {
				ctx.JSON(http.StatusInternalServerError, gin.H{"error": error.Error(err)})
			} else {
				if res.StatusCode != http.StatusOK {
					ctx.JSON(res.StatusCode, gin.H{"error": fmt.Sprintf("%s", data)})
				} else {
					ctx.JSON(res.StatusCode, gin.H{"token": data["id_token"].(string)})
				}
			}
		}
	}
}

// Update defines logic for Volunteer's account password reset
func (service AuthService) Update(ctx *gin.Context) {
	email := ctx.Query("email")
	if email == "" {
		ctx.JSON(http.StatusBadRequest, gin.H{"error": "email not provided"})
	} else {
		res, err := service.Api.UpdatePassword(email)
		if err != nil {
			ctx.JSON(http.StatusInternalServerError, gin.H{"error": error.Error(err)})
		} else {
			if res.StatusCode != http.StatusOK {
				data, err := service.readSingleResponse(res)
				if err != nil {
					ctx.JSON(http.StatusInternalServerError, gin.H{"error": error.Error(err)})
				} else {
					ctx.JSON(res.StatusCode, gin.H{"error": fmt.Sprintf("%s", data)})
				}
			} else {
				ctx.JSON(http.StatusOK, gin.H{"email": email})
			}
		}
	}
}

// Delete defines logic for Volunteer's account deletion
func (service AuthService) Delete(ctx *gin.Context) {
	authorized := CheckAuthorized(ctx)
	if authorized {
		email := ctx.Query("email")
		if email == "" {
			ctx.JSON(http.StatusBadRequest, gin.H{"error": "email not provided"})
		} else {
			userId, ok := service.getUserByEmail(email, ctx)
			if ok {
				res, err := service.Api.Delete(userId)
				if err != nil {
					ctx.JSON(http.StatusInternalServerError, gin.H{"error": error.Error(err)})
				} else {
					if res.StatusCode != http.StatusNoContent {
						data, err := service.readSingleResponse(res)
						if err != nil {
							ctx.JSON(http.StatusInternalServerError, gin.H{"error": error.Error(err)})
						}
						ctx.JSON(res.StatusCode, gin.H{"error": fmt.Sprintf("%s", data)})
					} else {
						ctx.JSON(http.StatusOK, gin.H{"email": email})
					}
				}
			}
		}
	}
}

// getUserByEmail defines logic for searching Volunteer's account by email
func (service AuthService) getUserByEmail(email string, ctx *gin.Context) (userId string, ok bool) {
	res, err := service.Api.ReadByEmail(email)
	ok = false
	if err != nil {
		ctx.JSON(http.StatusInternalServerError, gin.H{"error": error.Error(err)})
	} else {
		var multipleData []map[string]interface{}
		multipleData, err = service.readMultipleResponse(res)
		if len(multipleData) == 0 {
			ctx.JSON(http.StatusNotFound, gin.H{"email": email})
		} else {
			ok = true
			userId = multipleData[0]["user_id"].(string)
		}
	}
	return
}
