package service

import (
	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"institutionMS/layers/entity"
	"institutionMS/layers/repository"
	"net/http"
)

// InstitutionService is a struct that defines a institution Service behaviour at service layer
type InstitutionService struct {
	Repository repository.Repository
}

// Create is a business logic function that handles Institution POST HTTP method
func (service InstitutionService) Create(ctx *gin.Context) {
	authorized := CheckAuthorized(ctx)
	if authorized {
		institution := new(entity.Institution)
		err := ctx.Bind(institution)
		if err != nil {
			ctx.JSON(http.StatusBadRequest, gin.H{"error": error.Error(err)})
		} else {
			id, err := service.Repository.Create(*institution)
			if err != nil {
				ctx.JSON(http.StatusInternalServerError, gin.H{"error": error.Error(err)})
			} else {
				ctx.JSON(http.StatusCreated, gin.H{"id": id})
			}
		}
	}
}


// Read is a business logic function that handles Institution GET HTTP method
func (service InstitutionService) Read(ctx *gin.Context) {
	authorized := CheckAuthorized(ctx)
	if authorized {
		email:= ctx.Query("email")
		if email == ""{
			ctx.JSON(http.StatusBadRequest, gin.H{"error": "Email not provided"})
		} else {
			volunteer, err := service.Repository.Read(email)
			if err != nil {
				ctx.JSON(http.StatusInternalServerError, gin.H{"error": error.Error(err)})
			} else if volunteer == nil {
				ctx.JSON(http.StatusNotFound, gin.H{"email": email})
			} else {
				ctx.JSON(http.StatusOK, volunteer)
			}
		}
	}
}




// Update is a business logic function that handles Institution PUT HTTP method
func (service InstitutionService) Update(ctx *gin.Context) {
	authorized := CheckAuthorized(ctx)
	if authorized {
		id, _ := ctx.Params.Get("id")
		objId, err := primitive.ObjectIDFromHex(id)
		if err != nil {
			ctx.JSON(http.StatusBadRequest, gin.H{"error": error.Error(err)})
		} else {
			institution := new(entity.Institution)
			err := ctx.Bind(institution)
			if err != nil {
				ctx.JSON(http.StatusBadRequest, gin.H{"error": error.Error(err)})
			} else {
				updated, err := service.Repository.Update(objId, institution)
				if err != nil {
					ctx.JSON(http.StatusInternalServerError, gin.H{"error": error.Error(err)})
				} else if updated == nil {
					ctx.JSON(http.StatusNotFound, gin.H{"id": id})
				} else {
					ctx.JSON(http.StatusOK, updated)
				}
			}
		}
	}
}

// Delete is a business logic function that handles Institution DELETE HTTP method
func (service InstitutionService) Delete(ctx *gin.Context) {
	authorized := CheckAuthorized(ctx)
	if authorized {
		id, _ := ctx.Params.Get("id")
		objId, err := primitive.ObjectIDFromHex(id)
		if err != nil {
			ctx.JSON(http.StatusBadRequest, gin.H{"error": error.Error(err)})
		} else {
			deleted, err := service.Repository.Delete(objId)
			if err != nil {
				ctx.JSON(http.StatusInternalServerError, gin.H{"error": err.Error()})
			} else if deleted == nil {
				ctx.JSON(http.StatusNotFound, gin.H{"id": id})
			} else {
				ctx.JSON(http.StatusOK, deleted)
			}
		}
	}
}

