package test

import (
	"bytes"
	"fmt"
	"institutionMS/builder"
	"log"
	"net/http"
	"net/http/httptest"
	"os"
	"testing"

	"github.com/gin-gonic/gin"
	"github.com/joho/godotenv"
	"github.com/stretchr/testify/assert"
)

var idCreatedByPost string
var institutionBody = `{"isValid":true,"name":"testUser","email":"testUser@email.com","description":"Test","location":{"city":"POA"}}`
var institutionExpectedBody = `{"id":"%s","isValid":true,"name":"testUser","email":"testUser@email.com","description":"Test","location":{"complement":"","number":0,"street":"","city":"POA","state":"","country":""}}`
var institutionUpdatedBody = `{"isValid":false,"name":"testUpdatedUser","email":"testUserUpdated@email.com","description":"TestUpdated","location":{"city":"SP"}}`
var institutionExpectedUpdatedBody = `{"id":"%s","isValid":false,"name":"testUpdatedUser","email":"testUserUpdated@email.com","description":"TestUpdated","location":{"complement":"","number":0,"street":"","city":"SP","state":"","country":""}}`


func setId(w *httptest.ResponseRecorder) {
	idCreatedByPost = w.Body.String()
	idCreatedByPost = idCreatedByPost[7:len(idCreatedByPost)-2]
}

// setupRouter is a private function that loads Gin router
func setupRouter() (ginRouter *gin.Engine) {
	err := godotenv.Load("../.env")

	if err != nil {
		log.Fatal("Error loading .env file")
	}

	os.Setenv("NO_AUTH", "true")

	ginRouter = gin.Default()
	appRouter := builder.CreateInstitutionRouter()
	appRouter.CreateRouter(ginRouter)
	return
}

// TestSuccessInstitutionPost tests Institution's POST method with success response
func TestSuccessInstitutionPost(t *testing.T) {
	router := setupRouter()

	json := []byte(institutionBody)

	w := httptest.NewRecorder()
	req, _ := http.NewRequest("POST", "/institutions", bytes.NewBuffer(json))
	req.Header.Add("Content-Type", "application/json")
	router.ServeHTTP(w, req)

	setId(w)

	assert.Equal(t, http.StatusCreated, w.Code)
	assert.Contains(t, w.Body.String(), "id")
}

// TestSuccessInstitutionGet tests Institution's GET method with success response
func TestSuccessInstitutionGet(t *testing.T) {
	router := setupRouter()

	w := httptest.NewRecorder()
	req, _ := http.NewRequest("GET", "/institutions?email=testUser@email.com", nil)
	router.ServeHTTP(w, req)

	assert.Equal(t, http.StatusOK, w.Code)
	assert.Equal(t, fmt.Sprintf(institutionExpectedBody, idCreatedByPost), w.Body.String())
}

// TestSuccessInstitutionPut tests Institution's PUT method with success response
func TestSuccessInstitutionPut(t *testing.T) {
	router := setupRouter()

	json := []byte(institutionUpdatedBody)

	w := httptest.NewRecorder()
	req, _ := http.NewRequest("PUT", "/institutions/"+idCreatedByPost, bytes.NewBuffer(json))
	req.Header.Add("Content-Type", "application/json")
	router.ServeHTTP(w, req)

	assert.Equal(t, http.StatusOK, w.Code)
	assert.Equal(t, fmt.Sprintf(institutionExpectedUpdatedBody, idCreatedByPost), w.Body.String())
}

// TestSuccessInstitutionDelete tests Institution's DELETE method with success response
func TestSuccessInstitutionDelete(t *testing.T) {
	router := setupRouter()

	w := httptest.NewRecorder()
	req, _ := http.NewRequest("DELETE", "/institutions/"+idCreatedByPost, nil)
	req.Header.Add("Content-Type", "application/json")
	router.ServeHTTP(w, req)

	assert.Equal(t, http.StatusOK, w.Code)
	assert.Equal(t, fmt.Sprintf(institutionExpectedUpdatedBody, idCreatedByPost), w.Body.String())

	w = httptest.NewRecorder()
	req, _ = http.NewRequest("GET", "/institutions/"+idCreatedByPost, nil)
	req.Header.Add("Content-Type", "application/json")
	router.ServeHTTP(w, req)

	assert.Equal(t, http.StatusNotFound, w.Code)
}